program scleaner;

{$mode objfpc}{$H+}
{$macro on}

uses
  Classes, SysUtils, CustApp, FileUtil,
  exStrings;

type

  { TCleanSourcesApp }

  TCleanSourcesApp = class(TCustomApplication)
  protected
    directory: string;
    mask: string;
    details: boolean;
    erasedCount: integer;
    erasedSize: qword;
    procedure DoRun; override;
    function Clean(const aPath: string; const alvl: integer = 0): boolean;
  public
    constructor Create(TheOwner: TComponent); override;
    destructor Destroy; override;
    procedure WriteHelp; virtual;
  end;

{ TCleanSourcesApp }

procedure TCleanSourcesApp.DoRun;
begin
  WriteLN(' -- Cleansources by hinstance --- ');
  if HasOption('h','help') then begin
    WriteHelp;
    Terminate;
    Exit;
  end;

  details:=HasOption('details');
  if details then WriteLN('Details enabled');

  directory:=self.Location;
  if HasOption('d', 'directory') then
    directory:=self.GetOptionValue('d', 'directory');
  directory:=IncludeTrailingBackslash(directory);
  WriteLN('Dir: "', directory, '"');

  if not DirectoryExists(directory) then begin
    exitCode:=-1;
    WriteLN('Error: directory does not exists');
    Terminate;
    exit;
  end;

  mask:='*.o';
  if HasOption('s', 'sign') then
    mask:=self.GetOptionValue('s', 'sign');
  WriteLN('Sign: "' + mask + '"');

  erasedCount:=0;
  erasedSize:=0;
  Clean(directory);
  WriteLN('Files erased: ', erasedCount);
  WriteLN('Space: ', erasedSize / 1000 :3:0, 'k');

  Terminate;
end;

function TCleanSourcesApp.Clean(const aPath: string;
  const alvl: integer): boolean;

  procedure wlvl(const s: string);
  var
    i: integer;
  begin
    if not details then exit;
    for i:=0 to alvl do Write(' ');
    WriteLN(s);
  end;

var
  sr: TSearchRec;
  ss: TStringList;
  i: integer;
  sz: qword;
begin
  {$define fn:=aPath + sr.name}

  wlvl(ExtractFileName(ExcludeTrailingBackslash(aPath)) + ' - cleaning...');

  if FindFirst(aPath + '*', faAnyFile or faDirectory, sr) = 0 then
  repeat
    if (sr.Name = '.')or(sr.Name = '..') then continue;
    if DirectoryExists(fn) then Clean(fn + PathDelim, alvl + 1);
  until FindNext(sr)<>0;
  FindClose(sr);

  ss:=SplitString(mask, ';');
  for i:=0 to ss.Count - 1 do
  begin
    if FindFirst(aPath + ss[i], faAnyFile, sr) = 0 then
    repeat
      if (sr.Name = '.')or(sr.Name = '..') then continue;
      if not DirectoryExists(fn) then
      begin
        wlvl('-' + sr.name + ' - erasing...');
        sz:=FileSize(fn);
        if DeleteFile(fn) then begin
          inc(erasedCount);
          inc(erasedSize, sz);
        end;
      end;
    until FindNext(sr)<>0;
    FindClose(sr);
  end;

  {$undef fn}
end;

constructor TCleanSourcesApp.Create(TheOwner: TComponent);
begin
  inherited Create(TheOwner);
  StopOnException:=True;
end;

destructor TCleanSourcesApp.Destroy;
begin
  inherited Destroy;
end;

procedure TCleanSourcesApp.WriteHelp;
begin
  WriteLN('--directory=C:\MySources');
  WriteLN('--sign=*.myext1;*.myext2');
end;

var
  Application: TCleanSourcesApp;

{$IFDEF WINDOWS}{$R scleaner.rc}{$ENDIF}

begin
  Application:=TCleanSourcesApp.Create(nil);
  Application.Title:='Clean sources';
  Application.Run;
  Application.Free;
end.

