{ This file is a part of EPCL

  Copyright (C) 2009 Hinstance savyhinst@yandex.ru

  This source is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.

  This code is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.

  A copy of the GNU General Public License is available on the World Wide Web
  at <http://www.gnu.org/copyleft/gpl.html>. You can also obtain it by writing
  to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
  MA 02111-1307, USA.
}

unit exList;

{$mode objfpc}{$H+}
{$macro on}

{$define qSortFaster}

interface

uses
  Classes,
  SysUtils,
  exStrings, exListFace, exVariantList;

type

  { TGenericList }

  {$ifndef nodesign}
    TItems = array of integer;
    TCompareMethod = procedure;
    TCompareFunction = procedure;
    TCustomSortFunc = procedure;
    TNewFunction = procedure;
    TDestroyFunction = procedure;
    TItemAsString = procedure;
  {$endif}

  generic TGenericList<TItem> = class(TComponent, IListFace, IStringConvable)
      {Create a new TGenericList}
    constructor Create(const aOwner: TComponent = nil); reintroduce;
  protected
    function Construct: boolean; virtual;
  private
    {$ifdef nodesign}
    type TItems = array of TItem;
    TCompareFunction = function(const ax1, ax2: TItem): integer;
    TCompareMethod = function(const ax1, ax2: TItem): integer of object;
    TCustomSortFunc = function(const ax1, ax2: TItem;
      const args: TVariantList): integer;
    TNewFunction = function: TItem;
    TDestroyFunction = function(var aItem: TItem): boolean;
    TItemAsString = function(const aItem: TItem): string;
    //TDestroyMethod = function: boolean of object;
    var
    {$endif}
    fItems: TItems;
    fCount: integer;
    fCapacityReserve: integer;
    fSortingF: TCompareFunction;
    fSortingM: TCompareMethod;
    fCustomSortF: TCustomSortFunc;
    fNewFunc: TNewFunction;
    fDestroyF: TDestroyFunction;
    fItemAsString: TItemAsString;
    procedure clean;

    function GetCount: integer;
    function GetCapacity: integer;
    procedure SetCapacity(const aValue: integer);
    function GetLast: TItem;
    procedure SetLast(const aValue: TItem);
    function GetItem(const aIndex: integer): TItem;
    procedure SetItem(const aIndex: integer; const aValue: TItem);
    function getItemExists(const aIndex: integer): boolean; inline;

    function Compare(const ax1, ax2: TItem): integer;
    procedure JustExchange(const aIdx1, aIdx2: integer);
    procedure QuickSort(L, R: Integer);
    procedure QuickSort(L, R: Integer; const args: TVariantList);
    function GetFirst: TItem;
    function getItemAsText(const aIndex: integer): string;
    procedure SetFirst(const AValue: TItem);
    function GetAsText(const aFormat: TListTextOptions): string;
    function GetAsString: string;
  public
      {aOwner, CompareFunction, NewFunction, DestroyFunction}
    constructor Create(const aOwner: TComponent;
      const aNf: TNewFunction; const aCf: TCompareFunction;
      const aDf: TDestroyFunction);

    constructor Create(const aOwner: TComponent; const aItems: array of TItem);

    constructor Create(const aItems: array of TItem);

      {Capacity}
    property Capacity: integer read GetCapacity write SetCapacity;

      {Capacity increasion
      note: Capacity checked and works correctly}
    property CapacityReserve: integer
      read fCapacityReserve write fCapacityReserve;

    function UnCapacity: integer;

      {Set before sort}
    property SortingFunction: TCompareFunction read fSortingF write fSortingF;

      {Set before sort}
    property SortingMethod: TCompareMethod read fSortingM write fSortingM;

      {Set before sort() }
    property CustomSorting: TCustomSortFunc read fCustomSortF
      write fCustomSortF;

      {Set before AddNew}
    property NewFunction: TNewFunction read fNewFunc write fNewFunc;

      {SetBefore AsText}
    property TextFunction: TItemAsString
      read fItemAsString write fItemAsString;

      {Set before destroying or deleting; OPTIONAL}
    property DestroyFunction: TDestroyFunction read fDestroyF write fDestroyF;

    (*
      {Set before destroying or deleting; OPTIONAL}
    property DestroyMethod: TDestroyMethod read fDestroyM write fDestroyM;
    *)

      {Set NewFunction, CompareFunction, DestroyFunction}
    function SetMethods(const aNf: TNewFunction;
      const aCf: TCompareFunction;
      const aDf: TDestroyFunction): boolean;

      {Items}
    property Items[const aIndex: integer]: TItem
      read GetItem write SetItem; default;

      {Use it wisely}
    property DirectItems: TItems read fItems;

      {Texts}
    property Texts[const aIndex: integer]: string read GetItemAsText;

      {Count of items}
    property Count: integer read GetCount;

      {Returns true if the item is between [0 and count) }
    property ItemExists[const aIndex: integer]: boolean read getItemExists;

      {Add a new item; returns new item index}
    function Add(const aItem: TItem): integer;

      {Insert an item at place of item[aIndex]
        tested, seems to work correctly}
    function Insert(const aIndex: integer; const aItem: TItem): boolean;

      {Add a group of values}
    function Add(const aItems: array of TItem): integer;

      {Adds new elements x aCount}
    function AddNew(const aCount: integer = 1): boolean;

      {First item}
    property First: TItem read GetFirst write SetFirst;

      {Last item}
    property Last: TItem read GetLast write SetLast;

      {Exchanges items}
    function Exchange(const aIdx1, aIdx2: integer): boolean;

      {Sort using quicksort algorythm}
    function Sort: boolean;

      {Custom sort using quicksort algorythm, see tutorial,
        args will be free}
    function Sort(const args: TVariantList): boolean;

      {Custom sorting}
    function Sort(const args: array of variant): boolean;

      {applies "DestroyFunction" on item #aIndex}
    function DestroyItem(const aIndex: integer): boolean;

      {Delete an item and move all next items}
    function Delete(const aIndex: integer): boolean;

      {Returns true if all items was deleted succ. If
        the same item appears x2 or xX in list, result is false}
      {Important note:
        1 2 3 4 5 6 7 8 9
        Delete([7, 1, 3]);
        2 4 5 6 8 9 <=
      }
    function Delete(const aIndexes: array of integer): boolean;

      {Returns true if all the items destroyed True and DF assigned, o.c.}
    function DestroyItems: boolean;

      {Очистить}
    function Clear: boolean;

      {Replace items (if present) with aItems}
    function AssignItems(const aItems: TItems): boolean;

      {Returns list as text}
    function AsText(const aFormat: TListTextOptions = []): string;

      {Default CapacityReserve}
    function _DefaultCapacityReserve: integer; inline;

      //no comments
    destructor Destroy; override;
  end;

implementation

// const
 {$define const_msg_NilAsText := 'Nil TGenericList.' }
 {$define const_msg_emptyAsText := ': Empty list.' }
 {$define const_msg_textFuncNilItem := 'An item' }
 {$define const_msg_NoSuchItem := 'No such item' }

{ TGenericList }

{$define const_defaultCapacityReserve:=3 }
constructor TGenericList.Create(const aOwner: TComponent);
begin
  inherited Create(aOwner);
  Construct;
end;

function TGenericList.Construct: boolean;
begin
  clean;
  result:=true;
end;

constructor TGenericList.Create(const aOwner: TComponent;
  const aNf: TNewFunction; const aCf: TCompareFunction;
  const aDf: TDestroyFunction);
begin
  inherited Create(aOwner);
  Construct;
  SetMethods(aNf, aCf, aDf);
end;

constructor TGenericList.Create(const aOwner: TComponent;
  const aItems: array of TItem);
begin
  inherited Create(aOwner);
  Construct;
  Add(aItems);
end;

constructor TGenericList.Create(const aItems: array of TItem);
begin
  inherited Create(nil);
  Construct;
  Add(aItems);
end;

function TGenericList.UnCapacity: integer;
begin
  if self = nil then exit(0);
  result:=Capacity - Count;
  setLength(fItems, count);
end;

function TGenericList.SetMethods(const aNf: TNewFunction;
  const aCf: TCompareFunction; const aDf: TDestroyFunction): boolean;
begin
  if self = nil then exit(false);
  SortingFunction:=aCf;
  NewFunction:=aNf;
  DestroyFunction:=aDf;
  result:=true;
end;

procedure TGenericList.clean;
begin
  setLength(fItems, 0);
  fCount:=0;
  CapacityReserve:=_DefaultCapacityReserve;
  fSortingF:=nil;
  fSortingM:=nil;
  fCustomSortF:=nil;
  fNewFunc:=nil;
  fDestroyF:=nil;
  fItemAsString:=nil;
  //fDestroyM:=nil;
end;

function TGenericList.GetCount: integer;
begin
  result:=fCount;
end;

function TGenericList.GetCapacity: integer;
begin
  result:=length(fItems);
end;

procedure TGenericList.SetCapacity(const aValue: integer);
begin
  setLength(fItems, aValue);
end;

function TGenericList.GetLast: TItem;
begin
  if count = 0 then exit;
  result:=Items[Count - 1];
end;

procedure TGenericList.SetLast(const aValue: TItem);
begin
  if count = 0 then exit;
  Items[Count - 1]:=aValue;
end;

function TGenericList.GetItem(const aIndex: integer): TItem;
begin
  if ItemExists[aIndex] then result:=fItems[aIndex];
end;

procedure TGenericList.SetItem(const aIndex: integer; const aValue: TItem);
begin
  if ItemExists[aIndex] then fItems[aIndex]:=aValue;
end;

function TGenericList.getItemExists(const aIndex: integer): boolean;
begin
  result:=(0 <= aIndex)and(aIndex < Count);
end;

function TGenericList.Compare(const ax1, ax2: TItem): integer;
begin
  if SortingFunction<>nil then result:=SortingFunction(ax1, ax2);
  if SortingMethod<>nil then result:=SortingMethod(ax1, ax2);
end;

procedure TGenericList.JustExchange(const aIdx1, aIdx2: integer);
var
  b: TItem;
begin
  b:=fItems[aIdx1];
  fItems[aIdx1]:=fItems[aIdx2];
  fItems[aIdx2]:=b;
end;

procedure TGenericList.QuickSort(L, R: Integer);
var
  I, J, P: Integer;
  PivotItem: TItem;
begin
  repeat
    I := L;
    J := R;
    P := (L + R) div 2;
    repeat
      PivotItem := items[P];
      while Compare(PivotItem, items[I]) > 0 do
        Inc(I);
      while Compare(PivotItem, items[J]) < 0 do
        Dec(J);
      if I <= J then
      begin
        {$ifdef qSortFaster}
          JustExchange(I, J);
        {$else}
          Exchange(I, J);
        {$endif}
        if P = I then
          P := J
        else if P = J then
          P := I;
        Inc(I);
        Dec(J);
      end;
    until I > J;
    if L < J then
      QuickSort(L, J);
    L := I;
  until I >= R;
end;

procedure TGenericList.QuickSort(L, R: Integer; const args: TVariantList);
var
  I, J, P: Integer;
  PivotItem: TItem;
begin
  repeat
    I := L;
    J := R;
    P := (L + R) div 2;
    repeat
      PivotItem := items[P];
      while CustomSorting(PivotItem, items[I], args) > 0 do
        Inc(I);
      while CustomSorting(PivotItem, items[J], args) < 0 do
        Dec(J);
      if I <= J then
      begin
        {$ifdef qSortFaster}
          JustExchange(I, J);
        {$else}
          Exchange(I, J);
        {$endif}
        if P = I then
          P := J
        else if P = J then
          P := I;
        Inc(I);
        Dec(J);
      end;
    until I > J;
    if L < J then
      QuickSort(L, J, args);
    L := I;
  until I >= R;
end;

function TGenericList.GetFirst: TItem;
begin
  if count = 0 then exit;
  result:=items[0];
end;

function TGenericList.getItemAsText(const aIndex: integer): string;
begin
  if not assigned(TextFunction) then exit(const_msg_textFuncNilItem);
  if not ItemExists[aIndex] then
    exit(const_msg_NoSuchItem + ' (#' + AsString(aIndex) + ')');
  result:=TextFunction(items[aIndex]);
end;

procedure TGenericList.SetFirst(const AValue: TItem);
begin
  if count = 0 then exit;
  items[0]:=aValue;
end;

function TGenericList.GetAsText(const aFormat: TListTextOptions): string;
var
  i: integer;
begin
  if self = nil then exit(const_msg_NilAsText);
  if count = 0 then
    exit( AsString(self, true) + const_msg_emptyAsText);
  result:='';
  for i:=0 to Count - 1 do
  begin
    if tlNumbers in aFormat then
    case tlBRs in aFormat of
      true: result:=result + asString(i) + ': ';
      false: result:=result + asString(i) + ' = ';
    end;
    case tlQuotes in aFormat of
      true: result:=result + '"' + Texts[i] + '"';
      false: result:=result + Texts[i];
    end;
    case tlBRs in aFormat of
      true: result:=result + sLineBreak;
      false: if i <> (count - 1) then result:=result + ', ';
    end;
  end;
  if tlHead in aFormat then
  begin
    case tlBRs in aFormat of
      true: result:=sLineBreak + result;
      false: result:=': ' + result;
    end;
    result:=AsString(self, true) + AsString(['(', self.count, ')']) + result;
  end;
end;

function TGenericList.GetAsString: string;
begin
  result:=getAsText([]);
end;

function TGenericList.Add(const aItem: TItem): integer;
begin
  inc(fCount);
  while Capacity < Count do Capacity:=Capacity + CapacityReserve;
  fItems[Count - 1]:=aItem;
  result:=Count - 1;
end;

function TGenericList.Insert(const aIndex: integer;
  const aItem: TItem): boolean;
var
  i: integer;
begin
  Capacity:=Count + 1;
  fCount:=Count + 1;
  AddNew(1);
  for i:=count - 1 downto aIndex + 1 do items[i]:=items[i - 1];
  items[aIndex]:=aItem;
end;

function TGenericList.Add(const aItems: array of TItem): integer;
var
  i: integer;
begin
  Capacity:=count + length(aItems);
  result:=-1;
  for i:=0 to length(aItems) - 1 do result:=Add(aItems[i]);
end;

function TGenericList.AddNew(const aCount: integer): boolean;
var
  i: integer;
begin
  if NewFunction = nil then exit(false);
  result:=false;

  Capacity:=count + aCount;
  for i:=1 to aCount do Add(NewFunction());
  result:=true;
end;

function TGenericList.Exchange(const aIdx1, aIdx2: integer): boolean;
begin
  if self = nil then exit(false);
  result:=false;
  result:=ItemExists[aIdx1] and ItemExists[aIdx2];
  if not result then exit(false);
  JustExchange(aIdx1, aIdx2);
  result:=true;
end;

function TGenericList.Sort: boolean;
begin
  if self = nil then exit(false);
  if (SortingMethod = nil)and(SortingFunction = nil) then exit(false);
  result:=false;
  QuickSort(0, Count - 1);
  result:=true;
end;

function TGenericList.Sort(const args: TVariantList): boolean;
begin
  if self = nil then exit(false);
  if CustomSorting = nil then exit(false);
  args['glist']:=self;
  args['listowner']:=owner;
  QuickSort(0, Count - 1, args);
  args.Free;
  result:=true;
end;

function TGenericList.Sort(const args: array of variant): boolean;
begin
  Sort(TVariantList.Create(args));
end;

function TGenericList.DestroyItem(const aIndex: integer): boolean;
begin
  if self = nil then exit(false);
  if DestroyFunction = nil then exit(false);
  if not ItemExists[aIndex] then exit(false);
  result:=False;
  result:=DestroyFunction(fItems[aIndex]);
end;

function TGenericList.Delete(const aIndex: integer): boolean;
var
  i: integer;
begin
  if self = nil then exit(false);
  if not ItemExists[aIndex] then exit(false);
  case DestroyFunction = nil of
    false: result:=DestroyItem(aIndex);
    true: result:=true;
  end;
  for i:=aIndex + 1 to count - 1 do
    items[i - 1]:=items[i];
  dec(fCount);
end;

function TGenericList.Delete(const aIndexes: array of integer): boolean;
var
  i, ix, j: integer;
begin
  if self = nil then exit(false);
  result:=true;
  for i:=0 to length(aIndexes) - 1 do
  begin
    ix:=aIndexes[i];
    for j:=0 to i - 1 do
    begin
      if aIndexes[j]=aIndexes[i] then ix:=-1;
      if aIndexes[j]<ix then dec(ix);
    end;
    result:=result and Delete(ix);
  end;
end;

function TGenericList.DestroyItems: boolean;
var
  i: integer;
  r: boolean;
begin
  if not assigned(DestroyFunction) then exit(false);
  result:=false;
  r:=true;
  for i:=0 to count - 1 do r:=r and DestroyFunction(fItems[i]);
  result:=r;
end;

function TGenericList.Clear: boolean;
begin
  result:=self<>nil;
  if not result then exit;
  result:=false;
  case assigned(DestroyFunction) of
    true: result:=DestroyItems;
    false: result:=true;
  end;
  setLength(fItems, 0);
  fCount:=0;
end;

function TGenericList.AssignItems(const aItems: TItems): boolean;
begin
  result:=Clear;
  if not result then exit(false);
  Add(aItems);
end;

function TGenericList.AsText(const aFormat: TListTextOptions): string;
begin
  result:=GetAsText(aFormat);
end;

function TGenericList._DefaultCapacityReserve: integer;
begin
  result:=const_defaultCapacityReserve;
end;

destructor TGenericList.Destroy;
begin
  Clear;
  inherited Destroy;
end;

end.

