unit exLog;

{$mode objfpc}{$H+}
{$macro on}

interface

uses
  Classes, SysUtils,
  exStrings, exList, exStream;

type

  TOnLogText = function(const s: string): boolean of object;

  {$ifdef nodesign}
    TOnLogTexts = specialize TGenericList<TOnLogText>;
  {$else}
    TLogOnTexts = class(TList) end;
  {$endif}


  TLogModifier = (logTime, logDate, endl);

  { TCustomLog }

  TCustomLog = class(TComponent)
    constructor Create(const aOwner: TComponent); reintroduce;
    constructor Create(const aOwner: TComponent;
      const aOnTexts: array of TOnLogText);
  private
    fOnTexts: TOnLogTexts;
    procedure clean;
  protected
    function TextIt(const aText: string): boolean; virtual;
    function TextIt(const aWhat: array of const): boolean;
    function Initialize: boolean;
  public
    property OnTexts: TOnLogTexts read fOnTexts;
    destructor Destroy; override;
  end;

  { TStreamLog }

  TStreamLog = class(TCustomLog)
    constructor Create(const aOwner: TComponent; const aStream: TStream;
      const aFreeStream: boolean = false);
    constructor Create(const aOwner: TComponent; const aFileName: string);
    constructor Create(const aFileName: string);
    constructor CreateLocal(const aOwner: TComponent = nil);
  private
    fStream: TStream;
    fFreeStream: boolean;
    function getFileName: string;
  protected
    property FreeStream: boolean read fFreeStream;
    function TextIt(const aText: string): boolean; override;
  public
    property stream: TStream read fStream;
    property FileName: string read getFileName;
    class function LocalLogFN: string; inline;
    destructor Destroy; override;
  end;

operator + (const aLog: TCustomLog; const aString: string): TCustomLog;
operator + (const aLog: TCustomLog; const aInt: integer): TCustomLog;
operator + (const aLog: TCustomLog; const ax: int64): TCustomLog;
operator + (const aLog: TCustomLog; const ax: real): TCustomLog;
operator + (const aLog: TCustomLog; const ax: double): TCustomLog;
operator + (const aLog: TCustomLog; const ax: extended): TCustomLog;
operator + (const aLog: TCustomLog; const ax: boolean): TCustomLog;
operator + (const aLog: TCustomLog; const ax: TLogModifier): TCustomLog;
operator + (const aLog: TCustomLog; const ax: TObject): TCustomLog;
operator + (const aLog: TCustomLog; const a: array of const): TCustomLog;
{$ifdef nodesign}
operator << (const aLog: TCustomLog; const aString: string): TCustomLog;
operator << (const aLog: TCustomLog; const aInt: integer): TCustomLog;
operator << (const aLog: TCustomLog; const ax: real): TCustomLog;
operator << (const aLog: TCustomLog; const ax: double): TCustomLog;
operator << (const alog: TCustomLog; const ax: extended): TCustomLog;
operator << (const aLog: TCustomLog; const ax: TLogModifier): TCustomLog;
operator << (const aLog: TCustomLog; const ax: TObject): TCustomLog;
operator << (const aLog: TCustomLog; const a: array of const): TCustomLog;
{$endif}
operator - (const aLog: TCustomLog; const aObject: TObject): TCustomLog;

implementation

{ TCustomLog }

constructor TCustomLog.Create(const aOwner: TComponent);
begin
  inherited Create(aOwner);
  Initialize;
end;

constructor TCustomLog.Create(const aOwner: TComponent;
  const aOnTexts: array of TOnLogText);
begin
  Create(aOwner);
  OnTexts.Add(aOntexts);
end;

procedure TCustomLog.clean;
begin
  if self = nil then exit;
end;

function TCustomLog.TextIt(const aText: string): boolean;
var
  i: integer;
  r: boolean;
begin
  result:=false;
  if self = nil then exit;
  if OnTexts = nil then exit;
  r:=true;
  for i:=0 to OnTexts.Count - 1 do r:=r and OnTexts[i] (aText);
  result:=r;
end;

function TCustomLog.TextIt(const aWhat: array of const): boolean;
begin
  TextIt(AsString(aWhat));
end;

function TCustomLog.Initialize: boolean;
begin
  clean;
  fOnTexts:=TOnLogTexts.Create(self);
  result:=true;
end;

destructor TCustomLog.Destroy;
begin
  FreeAndNil(fOnTexts);
  inherited Destroy;
end;

operator + (const aLog: TCustomLog; const aString: string): TCustomLog;
begin
  aLog.TextIt(aString);
  result:=aLog;
end;

operator + (const aLog: TCustomLog; const aInt: integer): TCustomLog;
begin
  aLog.TextIt([aInt]);
  result:=aLog;
end;

operator + (const aLog: TCustomLog; const ax: int64): TCustomLog;
begin
  aLog.TextIt([ax]);
  result:=aLog;
end;

operator + (const aLog: TCustomLog; const ax: real): TCustomLog;
begin
  aLog.TextIt([ax]);
  result:=aLog;
end;

operator + (const aLog: TCustomLog; const ax: double): TCustomLog;
begin
  aLog.TextIt([ax]);
  result:=aLog;
end;

operator + (const alog: TCustomLog; const ax: extended): TCustomLog;
begin
  aLog.TextIt([ax]);
  result:=aLog;
end;

operator + (const aLog: TCustomLog; const ax: boolean): TCustomLog;
begin
  aLog.TextIt([ax]);
  result:=aLog;
end;

operator + (const aLog: TCustomLog; const ax: TLogModifier): TCustomLog;
begin
  case ax of
    logTime: aLog.TextIt(TimeToStr(Time));
    logDate: aLog.TextIt(DateToStr(Date));
    endl: aLog.TextIt(LineEnding);
  end;
  result:=aLog;
end;

operator + (const aLog: TCustomLog; const ax: TObject): TCustomLog;
begin
  aLog.TextIt([ax]);
  result:=aLog;
end;

operator + (const aLog: TCustomLog; const a: array of const): TCustomLog;
begin
  aLog.TextIt(a);
  result:=aLog;
end;

{$ifdef nodesign}

operator << (const aLog: TCustomLog; const aString: string): TCustomLog;
begin  result:=aLog + aString  end;

operator << (const aLog: TCustomLog; const aInt: integer): TCustomLog;
begin result:=aLog + aInt end;

operator << (const aLog: TCustomLog; const ax: real): TCustomLog;
begin result:=aLog + ax; end;

operator << (const aLog: TCustomLog; const ax: double): TCustomLog;
begin result:=aLog + ax; end;

operator << (const alog: TCustomLog; const ax: extended): TCustomLog;
begin result:=aLog + ax; end;

operator << (const aLog: TCustomLog; const ax: TLogModifier): TCustomLog;
begin result:=aLog + ax end;

operator << (const aLog: TCustomLog; const ax: TObject): TCustomLog;
begin result:=aLog + ax; end;

operator << (const aLog: TCustomLog; const a: array of const): TCustomLog;
begin result:=aLog + a end;
{$endif}

operator - (const aLog: TCustomLog; const aObject: TObject): TCustomLog;
begin
  aLog.TextIt(AsString(aObject, true) + ': ');
  result:=aLog;
end;

{ TStreamLog }

constructor TStreamLog.Create(const aOwner: TComponent; const aStream: TStream;
  const aFreeStream: boolean);
begin
  inherited Create(aOwner, []);
  fStream:=aStream;
  fFreeStream:=aFreeStream;
end;

constructor TStreamLog.Create(const aOwner: TComponent;
  const aFileName: string);
begin
  Create(aOwner, TFileStream.Create(aFileName,
    fmCreate or fmOpenWrite, fmShareDenyWrite), true);
end;

constructor TStreamLog.Create(const aFileName: string);
begin
  Create(nil, aFileName);
end;

constructor TStreamLog.CreateLocal(const aOwner: TComponent);
var
  fn: string;
begin
  fn:=GetAppConfigDir(false);
  if not DirectoryExists(fn) then ForceDirectories(fn);
  fn:=IncludeTrailingBackslash(fn);
  fn:=fn + LocalLogFN;
  Create(aOwner, fn);
end;

function TStreamLog.getFileName: string;
begin
  if self = nil then exit('');
  if stream = nil then exit('');
  if not (stream is TFileStream) then exit('');
  result:=(stream as TFileStream).FileName;
end;

function TStreamLog.TextIt(const aText: string): boolean;
begin
  if self = nil then exit(false);
  if fStream = nil then exit(false);
  inherited TextIt(aText);
  WriteStreamText(stream, aText);
  result:=true;
end;

class function TStreamLog.LocalLogFN: string; inline;
begin
  result:='log.text';
end;

destructor TStreamLog.Destroy;
begin
  if FreeStream then FreeAndNil(fStream);
  inherited Destroy;
end;

end.

