unit exSort;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils;

type

  { TQuickSort }

  generic TQuickSort<TItem> = class
  {$ifdef nodesign}
  private type
    TGetMethod = function (const aIdx: integer): TItem of object;
    TSetMethod = procedure (const aIdx: integer; const aValue: TItem) of object;
    TCompareMethod = function(const ax1, ax2: TItem): integer of object;
    var
  {$endif}
  public
    constructor Create(
      const aGetMeth: TGetMethod;
      const aSetMeth: TSetMethod;
      const aCompareMeth: TCompareMethod);
      reintroduce;
  private
    fGetMeth: TGetMethod;
    fSetMeth: TSetMethod;
    fCompareM: TCompareMethod;
    function GetItem(const aIndex: integer): TItem;
    procedure SetItem(const aIndex: integer; const aItem: TItem);
    property items[const i: integer]: TItem read GetItem write SetItem;
    procedure Exchange(const aIdx1, aIdx2: integer);
    function Compare(const ax1, ax2: TItem): integer;
      {То же самое, что и в RTL}
    procedure QuickSort(L, R: Integer);
  public
    procedure Sort(const aCount: integer);
  end;

implementation

{ TQuickSort }

constructor TQuickSort.Create(const aGetMeth: TGetMethod;
  const aSetMeth: TSetMethod; const aCompareMeth: TCompareMethod);
begin
  fGetMeth:=aGetMeth;
  fSetMeth:=aSetMeth;
  fCompareM:=aCompareMeth;
end;

function TQuickSort.GetItem(const aIndex: integer): TItem;
begin
  result:=fGetMeth(aIndex);
end;

procedure TQuickSort.SetItem(const aIndex: integer; const aItem: TItem);
begin
  fSetMeth(aIndex, aItem);
end;

procedure TQuickSort.Exchange(const aIdx1, aIdx2: integer);
var
  b: TItem;
begin
  b:=items[aIdx1];
  items[aIdx1]:=items[aIdx2];
  items[aIdx2]:=b;
end;

function TQuickSort.Compare(const ax1, ax2: TItem): integer;
begin
  result:=fCompareM(ax1, ax2);
end;

procedure TQuickSort.QuickSort(L, R: Integer);
var
  I, J, P: Integer;
  PivotItem: TItem;
begin
  repeat
    I := L;
    J := R;
    P := (L + R) div 2;
    repeat
      PivotItem := items[P];
      while Compare(PivotItem, items[I]) > 0 do
        Inc(I);
      while Compare(PivotItem, items[J]) < 0 do
        Dec(J);
      if I <= J then
      begin
        Exchange(I, J);
        if P = I then
          P := J
        else if P = J then
          P := I;
        Inc(I);
        Dec(J);
      end;
    until I > J;
    if L < J then
      QuickSort(L, J);
    L := I;
  until I >= R;
end;

procedure TQuickSort.Sort(const aCount: integer);
begin
  QuickSort(0, aCount - 1);
end;

end.

