{ This file is a part of EPCL

  Copyright (C) 2009 Hinstance savyhinst@yandex.ru

  This source is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.

  This code is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.

  A copy of the GNU General Public License is available on the World Wide Web
  at <http://www.gnu.org/copyleft/gpl.html>. You can also obtain it by writing
  to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
  MA 02111-1307, USA.
}


unit exStrings;

{$mode objfpc}{$H+}
{'macro on}

interface

uses
  Classes, SysUtils, Variants;

type
  IStringConvable = interface
    ['{A4C1030A-8796-455B-8648-36ECE4E50582}']
    function getAsString: string;
  end;

    { tlNames is for TVariantList }
  TListTextOption = (tlNumbers, tlQuotes, tlBRs, tlHead, tlNames);
  TListTextOptions = set of TListTextOption;

function asString(const aInteger: integer): string;
function asString(const aFloat: real): string;
function asString(const aBoolean: boolean): string;
function asString(const aChar: char): string;
function asString(const aChar: WideChar): string;
function asString(const aExtended: Extended): string;
function asString(const aString: string): string; inline;
function asString(const aPointer: pointer): string;
function asString(const aPChar: PChar): string;
function asString(const aVariant: Variant): string;
function asString(const aClass: TClass): string;
function asString(const aObject: TObject;
  const aObjhead:boolean = false): string;
function asString(const aException: Exception): string;

function asString(const aValue: TVarRec): string;
function asString(a: array of const): string;

function SplitString(const aStr: string; const aSymb: char = ' '): TStringList;

function asReal(const aStr: string): real;

const
  //const_erroneousRealStr = -666.666;
  const_erroneousRealStr = -1; //asReal(string): real erresult

implementation

const
  _PointerCharPrefix = '@';
  _NilObject = 'Nil';

function asString(const aInteger: integer): string;
begin
  result:=IntToStr(aInteger);
end;

function asString(const aFloat: real): string;
begin
  result:=FloatToStr(aFloat);
end;

function asString(const aBoolean: boolean): string;
begin
  result:=BoolToStr(aBoolean, true);
end;

function asString(const aChar: char): string;
begin
  result:=aChar;
end;

function asString(const aChar: WideChar): string;
begin
  result:=aChar;
end;

function asString(const aExtended: Extended): string;
begin
  result:=FloatToStr(aExtended);
end;

function asString(const aString: string): string; inline;
begin
  result:=aString;
end;

function asString(const aPointer: pointer): string; overload;
begin
  result:=_PointerCharPrefix + IntToStr(longint(aPointer));
end;

function asString(const aPChar: PChar): string;
begin
  result:=string(aPChar);
end;

function asString(const aVariant: Variant): string;
begin
  result:=VarToStr(aVariant);
end;

function asString(const aClass: TClass): string;
begin
  result:=aClass.ClassName;
end;

function asString(const aObject: TObject; const aObjhead: boolean): string;
var
  sface: IStringConvable;
begin
  if aObject = nil then exit( _NilObject );
  case aObject.GetInterface(IStringConvable, sface) and (not aObjhead) of
    true: begin
      result:=sface.getAsString;
      sface:=nil;
    end;
    false: begin
      result:=aObject.ClassName;
      if aObject is TComponent then
        if (aObject as TComponent).Name<>'' then
          result:=result + ' ' + (aObject as TComponent).Name;
    end;
  end;
end;

function asString(const aException: Exception): string;
begin
  result:=asString(aException.ClassType) + aException.Message;
end;

function asString(const aValue: TVarRec): string;
begin
  case aValue.VType of
    vtInteger: result:=asString(aValue.VInteger);
    vtBoolean: result:=asString(aValue.VBoolean);
    vtChar: result:=asString(aValue.VChar);
    vtWideChar: result:=asString(aValue.VWideChar);
    vtPWideChar: result:=asString(aValue.VPWideChar);
    vtAnsiString: result:=asString(PChar(aValue.VAnsiString));
    vtVariant: result:=asString(aValue.VVariant^);
    vtExtended: result:=asString(aValue.VExtended^);
    vtString: result:=asString(aValue.VString^);
    vtPointer: result:=asString(aValue.VPointer);
    vtPChar: result:=asString(aValue.VPChar);
    vtObject: result:=asString(aValue.VObject);
  end;
end;

function asString(a: array of const): string;
var
  i: integer;
begin
  result:='';
  for i:=0 to length(a) - 1 do
  begin
    result:=result + asString(a[i]);
  end;
end;

function SplitString(const aStr: string; const aSymb: char): TStringList;
begin
  result:=TStringList.Create;
  result.Delimiter:=aSymb;
  result.DelimitedText:=aStr;
end;

function asReal(const aStr: string): real;
begin
  result:=const_erroneousRealStr;
  if not TryStrToFloat(aStr, result) then result:=const_erroneousRealStr;
end;


end.



